// OAZipCodeFormatter:OATextFieldFormatter
//
// COPYRIGHT 1994 BY M. ONYSCHUK AND ASSOCIATES INC.
// ALL RIGHTS RESERVED.
//

#import "OAZipCodeFormatter.h"
#import "OATextFieldCell.h"

@implementation OAZipCodeFormatter:OATextFieldFormatter

- init
{
    [super init];
    
 // default behavior is to accept both U.S. and Canadian ZIP codes.
    americanFormatter = YES;
    canadianFormatter = YES;
    
    return self;
}


- filterSubstring:(unsigned char *)inputText
    ofSize:(int)inputSize
    at:(int)position
{
    if (NXIsAlpha(*inputText) &&
	canadianFormatter &&
	(position == 0 || position == 2 || position == 4)) {
	return[self setOutputFormat:"%c", *inputText];
    } else if (NXIsDigit(*inputText) &&
	       americanFormatter &&
	       (position == 0 || position == 2 || position == 4)) {
	return[self setOutputFormat:"%c", *inputText];
    } else if (NXIsDigit(*inputText) &&
	       !(position == 0 || position == 2 || position == 4)) {
	return[self setOutputFormat:"%c", *inputText];
    } else {
	NXBeep();
	return[self setOutput:NULL];
    }
}


- (BOOL)formatString:(unsigned char *)inputText
{
    BOOL                sensedCanadianCode = NO;
    int                 inputLen = (inputText) ? strlen(inputText) : 0;
     
 // Canadian postal codes always begin with an alphabetic character
    if (NXIsAlpha(inputText[0])) {
     	sensedCanadianCode = YES;
    }

    if (sensedCanadianCode) {
	int i;
	
    	if (!canadianFormatter) {
		return YES;
	}
	if (inputLen != 6) {
		return YES;
	}

	for (i = 0; i < 6; i++) {
	    switch (i) {
	    case 0:
	    case 2:
	    case 4:
		if (NXIsDigit(inputText[i])) {
		    return YES;
		}
		break;
	    default:
		if (NXIsAlpha(inputText[i])) {
		    return YES;
		}
	    }
	}
    } else {
    	char *c;
    
    	if (inputLen != 5 && inputLen != 9) {
		return YES;
	}
	
	for (c = inputText; *c; c++) {
	    if (NXIsAlpha(*c)) {
		return YES;
	    }
	}
    }
    
    [self setOutput:inputText];
    return NO;
}

- setTextFieldCell:aCell
{
    [aCell setMinimumLength:5];
    [aCell setChecksMinimumLength:YES];

    [aCell setMaximumLength:9];
    [aCell setChecksMaximumLength:YES];

    [aCell setCaseMapping:MAP_TO_UPPER];
    
    return self;
}

- (BOOL)isAmericanFormatter
{
    return americanFormatter;
}
- setAmericanFormatter:(BOOL)aValue
{
    americanFormatter = aValue;
    return self;
}

- (BOOL)isCanadianFormatter
{
    return canadianFormatter;
}
- setCanadianFormatter:(BOOL)aValue
{
    canadianFormatter = aValue;
    return self;
}

- read:(NXTypedStream *)aStream
{
    [super read:aStream];
    
    NXReadType(aStream,@encode(BOOL),&americanFormatter);
    NXReadType(aStream,@encode(BOOL),&canadianFormatter);
    
    return self;
}

- write:(NXTypedStream *)aStream
{
    [super write:aStream];
    
    NXWriteType(aStream,@encode(BOOL),&americanFormatter);
    NXWriteType(aStream,@encode(BOOL),&canadianFormatter);
    
    return self;
}

@end
