// OATextFieldCell:TextFieldCell
// 
// COPYRIGHT 1994 BY M. ONYSCHUK AND ASSOCIATES INC.
// ALL RIGHTS RESERVED.

#import <appkit/TextFieldCell.h>
#import <appkit/Text.h>

typedef enum _OACaseMapping {
	MAP_TO_NONE	= 0,
	MAP_TO_UPPER	= 1,
	MAP_TO_LOWER	= 2,
	MAP_TO_PROPER	= 3,
	MAP_TO_STUDLY	= 4
} OACaseMapping;

typedef enum _OACharacterFiltering {
	ALLOW_ALL		= 0,
	ALLOW_LETTERS		= 1,
	ALLOW_NUMBERS		= 2,
	ALLOW_ALPHANUMERIC	= 3
} OACharacterFiltering;

@class OATextFieldFormatter;

@interface OATextFieldCell:TextFieldCell
{
    OACharacterFiltering characterFiltering;

    char		*filteringExceptions;
    
    OACaseMapping	caseMapping;
    BOOL		isSettingValue;	// flag we're executing a case map...

    BOOL		removeLeadingWhitespace;
    BOOL		removeTrailingWhitespace;
        
    int			minimumLength;
    int			maximumLength;
    BOOL		checkMinimumLength;
    BOOL		checkMaximumLength;

    BOOL		isNullable;
    BOOL		isMandatory;
    
    int			maximumValue;
    int			minimumValue;
    BOOL		checkMinimumValue;
    BOOL		checkMaximumValue;   
    
    NXTextFilterFunc    oldTextFilter;	// hold the original filter func
    
    OATextFieldFormatter *formatter;
}

// Instance methods
- initTextCell:(const char *)stringValue;

- select:(const NXRect *)aRect
    inView:controlView
    editor:textObj
    delegate:anObject
    start:(int)selStart
    length:(int)selLength;

- edit:(const NXRect *)aRect
    inView:controlView
    editor:textObj
    delegate:anObject
    event:(NXEvent *)theEvent;
    
- endEditing:anObject;

- (OACaseMapping)caseMapping;
- setCaseMapping:(OACaseMapping)aValue;

- (OACharacterFiltering)characterFiltering;
- setCharacterFiltering:(OACharacterFiltering)aValue;

- (const char *)filteringExceptions;
- setFilteringExceptions:(const char *)aValue;

- (BOOL)doesRemoveLeadingWhitespace;
- setRemovesLeadingWhitespace:(BOOL)aValue;

- (BOOL)doesRemoveTrailingWhitespace;
- setRemovesTrailingWhitespace:(BOOL)aValue;

- (int)minimumLength;
- setMinimumLength:(int)aValue;

- (BOOL)doesCheckMinimumLength;
- setChecksMinimumLength:(BOOL)aValue;

- (int)maximumLength;
- setMaximumLength:(int)aValue;

- (BOOL)doesCheckMaximumLength;
- setChecksMaximumLength:(BOOL)aValue;

- (BOOL)isNullable;
- setNullable:(BOOL)aValue;

- (BOOL)isMandatory;
- setMandatory:(BOOL)aValue;

- (int)minimumValue;
- setMinimumValue:(int)aValue;

- (BOOL)doesCheckMinimumValue;
- setChecksMinimumValue:(BOOL)aValue;

- (int)maximumValue;
- setMaximumValue:(int)aValue;

- (BOOL)doesCheckMaximumValue;
- setChecksMaximumValue:(BOOL)aValue;

- (OATextFieldFormatter *)textFieldFormatter;
- setTextFieldFormatter:(OATextFieldFormatter *)aValue;
- setTextFieldFormatterNoCopy:(OATextFieldFormatter *)aValue;

- read:(NXTypedStream *)aStream;
- write:(NXTypedStream *)aStream;

- copyFromZone:(NXZone *)aZone;

- awake;
- awakeFromNib;

@end
